import * as SOTKLogin from './login';
import * as SOTKCardsList from './cardsList';
import * as SOTKHistory from './history';
import * as SOTKInvoices from './invoices';
import { RequestInit } from 'node-fetch';
import { HTMLElement } from 'node-html-parser';
export declare type SOTKCredentials = {
    token?: string;
    csrfToken?: string;
    authToken?: string;
};
export declare type SOTKAccount = {
    root: HTMLElement;
    balance: {
        token: any;
        pid: any;
    };
    delCard: {
        token: any;
        pid: any;
    };
    addCard: {
        token: any;
        pid: any;
    };
    history: {
        token: any;
        pid: any;
    };
};
export declare type OperationResult = {
    /** JSON-parsed result of operation. May be unicode-decoded text, object or anything else */
    data: any;
    success: boolean;
    message: null | any;
    messages: null | any;
};
export declare class SOTKBase {
    credentials: SOTKCredentials;
    accountInfo: null | SOTKAccount;
    constructor();
    fetch(url: any, options?: RequestInit): Promise<import("node-fetch").Response>;
    getAccountInfo(useCache?: boolean): Promise<SOTKAccount>;
    runJSONOperation(body: {
        [key: string]: string;
    }): Promise<OperationResult>;
    runOperation(body: {
        [key: string]: string;
    }): Promise<string>;
    login: typeof SOTKLogin.login;
    logout: typeof SOTKLogin.logout;
    getCards: typeof SOTKCardsList.getCards;
    getCardInfo: typeof SOTKCardsList.getCardInfo;
    addCard: typeof SOTKCardsList.addCard;
    deleteCard: typeof SOTKCardsList.deleteCard;
    getHistory: typeof SOTKHistory.getHistory;
    createInvoice: typeof SOTKInvoices.createInvoice;
}
export default SOTKBase;
